<?php
/**
 * Admin UI, register CPT and meta
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Sales_Admin' ) ) {

    /**
     * Holler_Sales_Admin class
     *
     * @since       0.2.0
     */
    class Holler_Sales_Admin {

        /**
         * @var         Holler_Sales_Admin $instance The one true Holler_Sales_Admin
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Sales_Admin
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Sales_Admin();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Run action and filter hooks
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         *
         *
         */
        private function hooks() {

            add_filter( 'hwp_custom_settings_save', array( $this, 'maybe_hide_avatar' ) );

            add_action( 'admin_init', array( $this, 'automatic_updates') );

            add_action( 'admin_init', array( $this, 'maybe_activate_deactivate_license') );

            if( !class_exists( 'Holler_Pro_Admin' ) )
                add_action( 'admin_notices', array( $this, 'hwp_license_error_notice' ) );

            add_action( 'hwp_type_settings', array( $this, 'type_settings') );

            add_action( 'hwp_settings_page', array( $this, 'license_fields' ), 15 );

            add_filter( 'hwp_settings_array', array( $this, 'add_fomo_settings') );

            add_action( 'hwp_after_position_settings', array( $this, 'fomo_settings' ) );

            add_action( 'transition_post_status',  array( $this, 'save_default_meta' ), 10, 3 );

        }

        /**
         * Add fomo type
         *
         * @access      public
         * @since       0.2
         */
        public function type_settings( $id ) {

            ?>

            <label class="hwp-radio-withimage">
                <span class="text">FOMO</span>
                <img src="<?php echo Holler_Box_Sales_URL . '/assets/img/fomo-icon.png'; ?>" class="hwp-radio-image" />
                <input type="radio" name="hwp_type" value="fomo" <?php checked( "fomo", get_post_meta( $id, 'hwp_type', 1 ) ); ?> />
            </label>

            <?php
        }

        /**
         * Add misc fomo settings
         *
         * @access      public
         * @since       0.4
         */
        public function fomo_settings( $id ) {

            // detect integrations
            if( class_exists('Easy_Digital_Downloads') )
                $edd = true;

            if( class_exists('WooCommerce') )
                $woo = true;

            if( class_exists('Give') )
                $give = true;

            if( is_plugin_active('memberpress/memberpress.php') )
                $memberpress = true;

            if( is_plugin_active('paid-memberships-pro/paid-memberships-pro.php') )
                $pmp = true;

            ?>

            <div class="hwp-section" id="fomo-settings" style="display:none">

                <h4>
                    <label for="fomo_integration"><?php _e( 'FOMO Integration' ); ?></label>
                </h4>

                <select name="fomo_integration">

                    <?php if( $edd === true ) : ?>
                        
                        <option value="edd" <?php selected( get_post_meta( $id, 'fomo_integration', true ), "edd"); ?> >
                        <?php _e( 'Easy Digital Downloads Sale', 'holler-box' ); ?>
                        </option>

                    <?php endif; ?>
                    
                    <?php if( $woo === true ) : ?>
                        
                        <option value="woo" <?php selected( get_post_meta( $id, 'fomo_integration', true ), "woo"); ?> >
                        <?php _e( 'WooCommerce Sale', 'holler-box' ); ?>
                        </option>

                    <?php endif; ?>

                    <?php if( $give === true ) : ?>
                        
                        <option value="give" <?php selected( get_post_meta( $id, 'fomo_integration', true ), "give"); ?> >
                        <?php _e( 'GiveWP Donation', 'holler-box' ); ?>
                        </option>

                    <?php endif; ?>

                    <?php if( $memberpress === true ) : ?>
                        
                        <option value="memberpress" <?php selected( get_post_meta( $id, 'fomo_integration', true ), "memberpress"); ?> >
                        <?php _e( 'MemberPress', 'holler-box' ); ?>
                        </option>

                    <?php endif; ?>

                    <?php if( $pmp === true ) : ?>
                        
                        <option value="pmp" <?php selected( get_post_meta( $id, 'fomo_integration', true ), "pmp"); ?> >
                        <?php _e( 'Paid Memberships Pro', 'holler-box' ); ?>
                        </option>

                    <?php endif; ?>

                </select>

                <h4>
                    <label for="fomo_image"><?php _e( 'FOMO Image' ); ?></label>
                </h4>

                <input type="radio" name="fomo_image" value="gravatar" <?php checked('gravatar', get_post_meta( $id, 'fomo_image', true ), true); ?>> Gravatar (uses customer email)<br>
                <input type="radio" name="fomo_image" value="product" <?php checked('product', get_post_meta( $id, 'fomo_image', true ), true); ?>> Product featured image<br>
                <!-- <input type="radio" name="fomo_image" value="map" <?php checked('map', get_post_meta( $id, 'fomo_image', true ), true); ?>> Map<br> -->

                <div id="fomo-time_ago-settings" style="display:none">

                    <h4>
                        <label for="fomo_time_ago"><?php _e( 'Timeago', 'hollerbox-sale' ); ?></label>
                    </h4>

                    <p><?php _e( 'Only show sales within this time period', 'hollerbox-sales' ); ?>

                    <?php $fomo_time_ago = ( get_post_meta( $id, 'fomo_time_ago', true ) ? get_post_meta( $id, 'fomo_time_ago', true ) : 1 ); ?>

                    <p><input type="number" class="hwp-number-input" name="fomo_time_ago" value="<?php echo intval( $fomo_time_ago ); ?>" min="1" /> days ago<br>

                </div>

                <h4>
                    <label for="fomo_settings"><?php _e( 'FOMO Settings' ); ?></label>
                </h4>

                <p><input type="number" class="hwp-number-input" name="fomo_loop_times" value="<?php echo intval( get_post_meta( $id, 'fomo_loop_times', true ) ); ?>" max="10" /> FOMOs per page<br>
                <input type="number" class="hwp-number-input" name="fomo_display_time" value="<?php echo intval( get_post_meta( $id, 'fomo_display_time', true ) ); ?>" /> display time<br>
                <input type="number" class="hwp-number-input" name="fomo_loop_delay" value="<?php echo intval( get_post_meta( $id, 'fomo_loop_delay', true ) ); ?>" /> delay between FOMOs</p>
                <input type="checkbox" id="fomo_order" name="fomo_order" value="1" <?php checked(1, get_post_meta( $id, 'fomo_order', true ), true); ?> /> Show sales in chronological order<br>

                <h4>
                    <label for="fomo_text"><?php _e( 'FOMO Text', 'hollerbox-sale' ); ?></label>
                </h4>

                <?php $fomo_text = ( get_post_meta( $id, 'fomo_text', true ) ? get_post_meta( $id, 'fomo_text', true ) : null ); ?>

                <p><input type="text" id="fomo_text" class="large-text" name="fomo_text" placeholder="{{name}} purchased {{product}}" value="<?php echo esc_html( $fomo_text ); ?>" /><br></p>

                <span class="description">
                    Text of FOMO, with placeholders. For example: {{name}} purchased {{product}}<br>
                    {{name}} - <?php _e( 'The name of the customer', 'hollerbox-sale' ); ?> <br>
                    {{product}} - <?php _e( 'The name of the product with link', 'hollerbox-sale' ); ?>
                </span>

            </div>

            <?php
        }

        /**
         * Hook into settings save. If someone creates a FOMO box and saves it, delete the avatar email so it doesn't show.
         *
         * @since       0.2.0
         * @return      void
         */
        public function maybe_hide_avatar( $id ) {

            // Check for type. If it's fomo, delete the avatar.
            $type = get_post_meta( $id, 'hwp_type', 1 );
            if( $type === 'fomo' )
                delete_post_meta( $id, 'avatar_email' );

        }

        /**
         * Initialize EDD Software Licensing
         *
         * @access      public
         * @since       0.2
         */
        public function automatic_updates() {

            // retrieve our license key from the DB
            $license_key = trim( get_option( 'hwp_fomo_license' ) );
            // setup the updater
            $edd_updater = new EDD_SL_Plugin_Updater( HWP_SALES_STORE_URL, Holler_Box_Sales_File, array( 
                    'version'   => Holler_Box_Sales_VER,       // current version number
                    'license'   => $license_key,    // license key (used get_option above to retrieve from DB)
                    'item_id'     => HWP_SALES_ID,    // id of this plugin
                    'author'    => 'Scott Bolinger',  // author of this plugin
                    'url'           => home_url(),
                    'beta'          => false
                )
            );

            // save our license settings
            self::edd_license_save();

        }

        /**
         * Add license key fields
         *
         * @access      public
         * @since       0.3
         */
        public function license_fields() {

            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option( 'hwp_fomo_license_status' );

            $class = '';

            if( $license_status !== false && $license_status == 'valid' ) 
                $class = 'hwp-license-active';

            ?>
            <style>input.hwp-license-active { border-bottom: 4px solid green; }</style>
            <table class="form-table">
            <tr valign="top">
                <th scope="row" valign="top">
                    <?php _e('FOMO license key'); ?>
                </th>
                <td>
                    <input id="hwp_fomo_license" name="hwp_fomo_license" type="text" class="regular-text <?php echo $class; ?>" value="<?php esc_attr_e( $license_key ); ?>" />
                    <?php if( $license_status !== false && $license_status == 'valid' ) echo '<br><small>License active</small>'; ?>
                </td>
            </tr>

            <?php if( false !== $license_key ) : ?>
                <tr valign="top">
                    <th scope="row" valign="top">
                        
                    </th>
                    <td>
                        <?php if( $license_status !== false && $license_status == 'valid' ) : ?>
                            <input type="submit" class="button-secondary" name="fomo_license_deactivate" value="<?php _e('Deactivate License'); ?>"/>
                        <?php else: ?>
                                <input type="submit" class="button-primary" name="fomo_license_activate" value="<?php _e('Activate License'); ?>"/>
                        <?php endif; ?>
                    </td>
                </tr>
            <?php endif; ?>
            </table>
            <?php
        }

        /**
         * Save license settings
         *
         * @access      public
         * @since       0.4
         */
        public function edd_license_save() {

            // run a quick security check
            // if ( !isset( $_POST['hollerbox_settings_nonce'] ) || !wp_verify_nonce( $_POST['hollerbox_settings_nonce'], basename( __FILE__ ) ) )
            // return;

            // don't run if we are deactivating licenses
            if( isset( $_POST['fomo_license_deactivate'] ) || isset( $_POST['pro_license_deactivate'] ) )
                return;

            /** Licensing **/
            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option('hwp_fomo_license_status');

            $postval = ( isset( $_POST['hwp_fomo_license'] ) ? $_POST['hwp_fomo_license'] : '' );

            if( $postval != '' ) {

                update_option( 'hwp_fomo_license', sanitize_text_field( $postval ) );

            } 

            if( isset( $_POST['hwp_fomo_license'] ) && $_POST['hwp_fomo_license'] === '' ) {
                // delete license stuff if field is blank
                delete_option( 'hwp_fomo_license' );
                delete_option( 'hwp_fomo_license_status' );

            }

        }

        /**
         * Maybe Deactivate license
         *
         * @access      public
         * @since       0.2
         */
        public function maybe_activate_deactivate_license() {

            // listen for our activate button to be clicked
            if( isset( $_POST['fomo_license_deactivate'] ) ) {

                $license_key = trim( get_option( 'hwp_fomo_license' ) );

                $res = $this->deactivate_license( $license_key, HWP_SALES_ID );

                // wp_die( $res );

                // $license_data->license will be either "deactivated" or "failed"
                if( $res == 'deactivated' ) {
                    delete_option( 'hwp_fomo_license_status' );
                    $redirect = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                } else {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res ) ), $base_url );
                }

                wp_redirect( $redirect );
            } else if( isset( $_POST['fomo_license_activate'] ) && isset( $_POST['hwp_fomo_license'] ) ) {

                $license_key = $_POST['hwp_fomo_license'];

                $res = $this->activate_license( $license_key, HWP_SALES_ID );

                // $res is any error messages from license activation. Display them to user on redirect if necessary.
                if ( ! empty( $res['msg'] ) ) {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res['msg'] ) ), $base_url );
                    wp_redirect( $redirect );
                    exit();
                }

                // $license_data->license will be either "valid" or "invalid"
                update_option( 'hwp_fomo_license_status', $res['status'] );

                wp_redirect( admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' ) );

            }
        }

        /**
         * Activate license key by calling EDD SL API
         *
         * @access      public
         * @since       0.2
         */
        public static function activate_license( $license, $item_id ) {

            $message = '';
 
            // data to send in our API request
            $api_params = array(
                'edd_action' => 'activate_license',
                'license'    => $license,
                'item_id'  => $item_id,
                'url'        => home_url()
            );

            // Call the custom API.
            $response = wp_remote_post( HWP_SALES_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );
            
            // make sure the response came back okay
            if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

                $message =  ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.' );
                $ldata = null;

            } elseif( !empty( wp_remote_retrieve_body( $response ) ) ) {

                $license_data = json_decode( wp_remote_retrieve_body( $response ) );
                if ( false === $license_data->success ) {
                    switch( $license_data->error ) {
                        case 'expired' :
                            $message = sprintf(
                                __( 'Your license key expired on %s.' ),
                                date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
                            );
                            break;
                        case 'revoked' :
                            $message = __( 'Your license key has been disabled.' );
                            break;
                        case 'missing' :
                            $message = __( 'Invalid license.' );
                            break;
                        case 'invalid' :
                        case 'site_inactive' :
                            $message = __( 'Your license is not active for this URL.' );
                            break;
                        case 'item_name_mismatch' :
                            $message = sprintf( __( 'This appears to be an invalid license key for %s.' ), HWP_SALES_ID );
                            break;
                        case 'no_activations_left':
                            $message = __( 'Your license key has reached its activation limit.' );
                            break;
                        default :
                            $message = __( 'An error occurred, please try again.' );
                            break;
                    }
                }

                $ldata  = $license_data->license;
            }

            return array( 'msg' => $message, 'status' => $ldata );

            exit();

        }

        /**
         * Deactivate license
         *
         * @access      public
         * @since       0.2
         */
        public static function deactivate_license( $license_key, $item_id ) {

            // data to send in our API request
            $api_params = array(
                'edd_action' => 'deactivate_license',
                'license'    => $license_key,
                'item_id'  => $item_id,
                'url'        => home_url()
            );

            // Call the custom API.
            $response = wp_remote_post( HWP_SALES_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

            // make sure the response came back okay
            if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

                if ( is_wp_error( $response ) ) {
                    $message = $response->get_error_message();
                } else {
                    $message = __( 'An error occurred, please try again.' );
                }

                $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

                wp_redirect( $redirect );
                exit();
            }

            // decode the license data
            $license_data = json_decode( wp_remote_retrieve_body( $response ) );

            return $license_data->license;

        }

        /**
         * Catch and display licensing errors
         *
         * @access      public
         * @since       0.2
         */
        public static function hwp_license_error_notice() {
            if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['message'] ) ) {
                switch( $_GET['sl_activation'] ) {
                    case 'false':
                        $message = urldecode( $_GET['message'] );
                        ?>
                        <div class="error">
                            <p><?php echo $message; ?></p>
                        </div>
                        <?php
                        break;
                    case 'true':
                    default:
                        // Developers can put a custom success message here for when activation is successful if they want.
                        break;
                }
            }
        }

        /**
         * Add any fomo meta keys here so they get saved
         *
         * @access      public
         * @since       0.4
         * @return array()
         */
        public function add_fomo_settings( $settings ) {

            $settings[] = 'fomo_integration';
            $settings[] = 'fomo_image';
            $settings[] = 'fomo_loop_times';
            $settings[] = 'fomo_loop_delay';
            $settings[] = 'fomo_display_time';
            $settings[] = 'fomo_order';
            $settings[] = 'fomo_time_ago';
            $settings[] = 'fomo_text';

            return $settings;
        }

        /**
         * Save meta box defaults when new post is created
         *
         * @param     string $new_status
         * @param     string $old_status
         * @param     WP_Post $post
         */
        public function save_default_meta( $new_status, $old_status, $post ) {
            
            if ( $old_status === 'new' && $new_status === 'auto-draft' && $post->post_type === 'hollerbox' ) {

                $post_id = $post->ID;

                $setting = get_post_meta( $post_id, 'fomo_loop_times' );

                // if we already have a setting, bail
                if( !empty( $setting ) )
                    return;

                // Check if the current user has permission to edit the post.
                if ( !current_user_can( 'edit_post', $post_id ) )
                    return;

                update_post_meta( $post_id, 'fomo_loop_times', 1 );
                update_post_meta( $post_id, 'fomo_display_time', 5 );
                update_post_meta( $post_id, 'fomo_loop_delay', 6 );
                update_post_meta( $post_id, 'fomo_time_ago', 14 );

            }

        }

    }


    $holler_sales_admin = new Holler_Sales_Admin();
    $holler_sales_admin->instance();

} // end class_exists check